#!/usr/bin/env python

#
#
#Copyright (C) 2010 BoscoWitch
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

# Import a few necessary components for the script to run
import screenlets
from screenlets.options import StringOption, BoolOption, IntOption, FontOption, ColorOption
import cairo
import pango
import gtk
import gobject
import datetime
import locale
import pango
import commands
from os import path
import os
import sys
import sqlite3

# Begin Creating the actual program by creating the class...MUST ACCEPT screenlets.Screenlet.
class RandomWadokuScreenlet(screenlets.Screenlet):
	"""A Sample Screenlet for Compiz"""

	# Default Meta Info for Screenlet
	__name__ = 'RandomWadoku'
	__version__ = '0.2'
	__author__ = 'BoscoWitch'
	__desc__ = 'Displays Random Wadoku Entry'

	# Internal Variables
	__timeout = None
	p_layout = None

	# editable options and defaults
	update_interval = 40
	show_romanji = False
	show_reading = True
	font = "sans 12"
	fontsize = 12
	text_color = ( 0,0,0,0.8 )
	mytext = "Loading..."
	install_path = None
	frame = ( 0, 0, 0, 0 )
	wadoku_db = None
	db_cursor = None
	
# Define the initialization function - Some variation on this __init__ function is necessary for all screenlets.
	def __init__(self, **keyword_args):
		# Call screenlets.Screenlet.__init__ => Required!
		# Declare the basic attributes of the screenlet - width, height
		screenlets.Screenlet.__init__(self, width=800, height=200, **keyword_args)
		#Get Path to RandomWadoku directory
		self.install_path = path.dirname(path.abspath(__file__)) + os.sep	
		sys.path.append(self.install_path)
		#Open wadoku sqlite3 Database
		self.wadoku_db = sqlite3.connect(self.install_path + 'wadoku_Database.sqlite3')
		self.db_cursor = self.wadoku_db.cursor()

		# Tells the screenlet which theme to use
		self.theme_name="default"
		# Create the "basic" pop-up menu that should be displayed when right-clicked
		self.add_default_menuitems()
		# Add an extra tab under the "options" tab on the "properties" dialog
		self.add_options_group('Anzeigen', 'Anzeige Optionen')
		# Add content to the new tab page that was created
		self.add_option(ColorOption('Anzeigen','frame', 
			self.frame, 'Hintergund Farbe', 'HIntergrund Farbe'))
		self.add_option(FontOption ('Anzeigen', 'font', self.font, 'Schriftart', 'Die Schriftart'))
		self.add_option(ColorOption('Anzeigen','text_color', 
			self.text_color, 'Text Farbe', 'Text Farbe'))
		self.add_option(IntOption('Anzeigen', 'update_interval', 
			self.update_interval, 'Update interval', 
			'The interval for updating info (in seconds)', min=5, max=600))
		self.add_option(BoolOption('Anzeigen', 'show_romanji',
			self.show_romanji, 'Zeige Romanji', 'Zeigt Romanji lesung in Romanji an'))
		self.add_option(BoolOption('Anzeigen', 'show_reading',
			self.show_reading, 'Zeige Lesung', 'Zeige Lesung in Hiragana an'))
		
		# Set the update interval (In this case, how often the canvas should be redrawn
		self.update_interval = self.update_interval
		self.update()

	# Define the __setattr__ function - This is also called when the script is executed
	def __setattr__(self, name, value):
		# Call screenlets.Screenlet.__setattr__ => Required!
		screenlets.Screenlet.__setattr__(self, name, value)
		# Set up a timer based on the "update_interval" value defined above
		if name == "update_interval":
			if value > 0:
				# Create a 'dictionary' with the update_interval value
				self.__dict__['update_interval'] = value
				# If the timer exists and has timed out, remove it
				if self.__timeout:
					gobject.source_remove(self.__timeout)
				# Create the timer with a value of value * 1000
				# Because the timer accepts a time value in milliseconds
				# Note that the object calls the function update() on timeout*
				self.__timeout = gobject.timeout_add(int(value * 1000), self.update)
			else:
				# If 'value' isn't set it, set it as 1 (second) in the 'dictionary'
				self.__dict__['update_interval'] = 1
				pass

	# Define the update function - in this case, what to do when the timer expires
	def update(self):
		self.db_cursor.execute('select * from entries ORDER BY RANDOM() LIMIT 1;')
		# Redraw the canvas
		row = self.db_cursor.fetchone()
		self.mytext = row[2] + '\n'
		if self.show_reading:
			self.mytext += row[3] + '\n'
		if self.show_romanji:
			self.mytext += row[4] + '\n'
		self.mytext += row[1]		
		self.redraw_canvas()
		return True

	# The 'on_draw' function tells the script how to draw the widget
	def on_draw(self, ctx):
		try : self.fontsize = int(self.font.strip().split(' ')[2])
		except:
			try: self.fontsize = int(self.font.strip().split(' ')[1])
			except : self.fontsize = 10
		# Make sure the scale value is within the acceptable range
		if self.scale > 5:
			self.scale = 5
		# If the theme is set (in our case, it is set as "default"), draw the screenlet
		if self.theme:
			ctx.save()
			ctx.scale(self.scale, self.scale)			
			ctx.set_source_rgba(*self.frame)
			self.draw_rectangle_advanced (ctx, 0, 0, self.width, self.height, fill=True, border_size=0)
			ctx.restore()

			# Draw the text onto the background
			# Depends on the 'draw_text()' function defined below
			ctx.save()
			ctx.scale(self.scale, self.scale)
			ctx.set_source_rgba(*self.text_color)
			self.draw_text(ctx, '' + self.mytext + '', 1, 1, 800, self.font.split(' ')[0], self.fontsize)
			ctx.restore()

	# This on_draw_shape provides for the majority of the functionality of the program
	# Without this function, you would be unable to "right click" on the widget (the menu
	# for the window behind it would appear) and you would be unable to move the screenlet
	# once it appears on the screen.
	def on_draw_shape(self, ctx):
		if self.theme:
			self.on_draw(ctx)

	# This is a user defined function that will tell python how to draw the text onto the background
	def draw_text(self, ctx, value, x, y, width, font, size):
		ctx.save()
		ctx.translate(x, y)
		# Create a layout for the text which will be drawn on the background later
		if self.p_layout == None:
			self.p_layout = ctx.create_layout()
		else:
			ctx.update_layout(self.p_layout)
		# The next four lines set up the font for the layout
		p_fdesc = pango.FontDescription()
		p_fdesc.set_family_static(font)
		p_fdesc.set_size(size * pango.SCALE)
		self.p_layout.set_font_description(p_fdesc)
		# Define how wide the text can be that is written to the screen, in pixels
		# I want a 10 pixel border on both sides, so we will set this as 150 - 2*10 = 130px
		self.p_layout.set_width(width * pango.SCALE)
		# Left-Align the text
		self.p_layout.set_alignment(pango.ALIGN_LEFT)
		self.p_layout.set_markup(value)
		# Set the color of the text
		# Setup: (RED, GREEN, BLUE, OPACITY) => All values go from 0 to 1 (float format)
		ctx.set_source_rgba(*self.text_color)
		# Display the text
		ctx.show_layout(self.p_layout)
		# Center the text on the background
		self.p_layout.set_alignment(pango.ALIGN_CENTER)
		ctx.restore()

# If the program is run directly or passed as an argument to the python
# interpreter then create a Screenlet instance and show it
# Essentially, initialize the program
if __name__ == "__main__":
	import screenlets.session
	screenlets.session.create_session(RandomWadokuScreenlet)
